public class Oops {

    /*
     * This example throws a java.lang.ArrayIndexOutOfBoundsException.
     */
    public void fail1()
    {
	try {
	    int[] x = { 1, 2, 3 };
	    int z = x[5];
	} catch(Exception e) {
	    System.err.println("fail1: " + e);
	}
    }

    /*
     * This example throws a java.lang.ArithmeticException.
     */
    public void fail2()
    {
	try {
	    int x = 1, y = 0;
	    int z = x/y;
	} catch(Exception e) {
	    System.err.println("fail2: " + e);
	}
    }

    /*
     * This example throws and catches a MyException.
     */
    public void fail3()
    {
	try {
	    throw new MyException("programmer was too lazy to implement this");
	} catch(MyException e) {
	    System.err.println("fail3: " + e.getClass().getName() + ": "
			       + e.getMessage());
	}
    }

    /*
     * This example throws and catches a MyException. Note that the
     * order of the catch handlers matters (put the most specific
     * first).
     */
    public void fail4()
    {
	try {
	    throw new MyException("programmer was too lazy to implement this");
	} catch(MyException e) {
	    System.err.println("fail4: MyException: " + e.getMessage());
	} catch(Exception e) {
	    System.err.println("fail4: Exception: " + e.getMessage());
	}
    }

    /*
     * This example shows how to conveniently print the stack trace
     * for debugging purposes (not to be used in final code).
     */
    public void fail5()
    {
	try {
	    throw null;
	} catch(Exception e) {
	    System.err.print("fail5: ");
	    e.printStackTrace();
	}
    }

    /*
     * This example shows how to print the stack trace manually
     * for debugging purposes (not to be used in final code).
     */
    public void fail6()
    {
	try {
	    throw null;
	} catch(Exception e) {
	    System.err.println("fail6: " + e.getClass().getName());
	    StackTraceElement[] frames = e.getStackTrace();
	    for (StackTraceElement f: frames) {
		System.err.println("        at " + f);
	    }
	}
    }
    
    /*
     * This example shows that the finally block is executed before
     * the value is returned to the caller. Hence, this example
     * returns -1 and not 1.
     */
    public int fail7()
    {
	try {
	    if (true) throw null;
	    return 0;
	} catch(Exception e) {
	    return 1;
	} finally {
	    return -1;
	}
    }

    /*
     * This example shows that the finally block is executed before
     * the value is returned to the caller. This example returns -1
     * and not 1. The outer try/catch is not evaluated since the
     * inner finally causes the value -1 to be returned.
     */
    public int fail8()
    {
	try {
	    try {
		if (true) throw null;
		return 0;
	    } finally {
		return -1;
	    }
	} catch(Exception e) {
	    return 1;
	}
    }

    public static void main(String[] args)
    {
	Oops o = new Oops();

	o.fail1();
	o.fail2();
	o.fail3();
	o.fail4();
	o.fail5();
	o.fail6();

	System.out.println("fail7: returning " + o.fail7());
	System.out.println("fail8: returning " + o.fail8());
    }
}
